// generated by Fast Light User Interface Designer (fluid) version 1.0303

#ifndef fast_slow_h
#define fast_slow_h
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Slider.H>
extern Fl_Slider *control;
extern Fl_Slider *fast;
extern Fl_Slider *slow;
#include <FL/Fl_Box.H>
#endif
