// generated by Fast Light User Interface Designer (fluid) version 1.0303

#ifndef radio_h
#define radio_h
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Light_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Output.H>
extern Fl_Output *cb_info;
#endif
